<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2018, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
/**
 * @author Ebenezer Obasi <info@eobasi.com>
 * @package ow_plugins.helpcenter.classes
 * @since 1.0
 */
class HELPCENTER_CLASS_CategoryField extends RadioField
{
	/**
     * @var integer
     */
    protected $columnCount;

    /**
     * Input options.
     *
     * @var array
     */
    protected $options;

    /**
     * Constructor.
     *
     * @param string $name
     */
    public function __construct( $name )
    {
        parent::__construct($name);
		
        $this->addAttribute('type', 'radio');
        $this->columnCount = 1;
    }

    /**
     * @see FormElement::renderInput()
     *
     * @param array $params
     * @return string
     */
    public function renderInput( $params = null )
    {
        parent::renderInput($params);
		
		$categories = HELPCENTER_BOL_Service::getInstance()->findAllCategory( true );

        $renderedString = '<ul class="ow_radio_group clearfix" style="height:200px;overflow-y:scroll;">';
		
		foreach($categories as $category)
		{
			$children = HELPCENTER_BOL_Service::getInstance()->findCategoryChildren( $category->getId( ) );
			
			if ( $this->value !== null && (string) $category->getId( ) === (string) $this->value )
			{
				$this->addAttribute(FormElement::ATTR_CHECKED, 'checked');
			}

			$this->setId(UTIL_HtmlTag::generateAutoId('input'));
			$this->addAttribute('value', $category->getId( ));
			
			$renderedString .= '<li style="width:100%">' . '<h3 for="' . $this->getId() . '">' . $category->getName( ) . '</h3></li>';
			$this->removeAttribute(FormElement::ATTR_CHECKED);
			
			if( !empty( $category->children ) )
			{				
				foreach($category->children as $child)
				{
					if ( $this->value !== null && (string) $child->getId( ) === (string) $this->value )
					{
						$this->addAttribute(FormElement::ATTR_CHECKED, 'checked');
					}
					
					$this->setId(UTIL_HtmlTag::generateAutoId('input'));
					$this->addAttribute('value', $child->getId( ));

					$renderedString .= '<li class="ow_small" style="width:80%;display:block;padding-left:10px;">' . UTIL_HtmlTag::generateTag('input', $this->attributes) . '&nbsp;<label for="' . $this->getId() . '">' . $child->getName( ) . '</label></li>';
					$this->removeAttribute(FormElement::ATTR_CHECKED);
					
					foreach($child->children as $grandChild)
					{
						if ( $this->value !== null && (string) $grandChild->getId( ) === (string) $this->value )
						{
							$this->addAttribute(FormElement::ATTR_CHECKED, 'checked');
						}
						
						$this->setId(UTIL_HtmlTag::generateAutoId('input'));
						$this->addAttribute('value', $grandChild->getId( ));

						$renderedString .= '<li class="ow_small" style="width:80%;display:block;padding-left:20px;">' . UTIL_HtmlTag::generateTag('input', $this->attributes) . '&nbsp;<label for="' . $this->getId() . '">' . $grandChild->getName( ) . '</label></li>';
						$this->removeAttribute(FormElement::ATTR_CHECKED);
					}
				}
			}
		}

        return $renderedString . '</ul>';
    }
}