<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product.
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2018, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Ebenezer Obasi <info@eobasi.com>
 * @package ow_plugins.helpcenter.classes
 * @since 1.0
 */
class HELPCENTER_CLASS_MasterPage extends OW_MasterPage
{
    /**
     * Constructor.
     */
    public function __construct( )
    {
		parent::__construct( );

        $this->init();
    }

	/**
     * @see OW_MasterPage::init()
     */
    protected function init()
    {
		parent::init();
		
		$service = HELPCENTER_BOL_Service::getInstance();
		$this->assign('displayFaq', (bool)$service->config->display_faq);

		$menu = new HELPCENTER_CMP_Menu();
		$this->addComponent('menu', $menu);
		
		$siteName = OW::getConfig()->getValue('base', 'site_name');
		$helpCenterUrl = OW::getRouter()->urlForRoute('helpcenter_view_index');		
		$helpCenterTitle = OW::getLanguage()->text('helpcenter', 'main_title', array('siteName' => $siteName));
		$siteUrl = OW::getRouter()->getBaseUrl();
		
		OW::getDocument()->setTitle($helpCenterTitle);
		
		$this->assign('helpCenterTitle', $helpCenterTitle);		
		$this->assign('helpCenterUrl', $helpCenterUrl);
		$this->assign('siteName', $siteName);		
		$this->assign('siteUrl', $siteUrl);

		$attrs = OW::getRequestHandler()->getHandlerAttributes();

		if ( $attrs[OW_RequestHandler::ATTRS_KEY_ACTION] == "index" )
		{
			$this->assign('homepage', true);
		}
	}

	private function loadStaticFiles()
	{
		if ( OW::getRequest()->isAjax())
		{
			return;
		}

		$document = OW::getDocument();
		$themeManager = OW::getThemeManager();
		$service = HELPCENTER_BOL_Service::getInstance();
		
        $pluginStaticUrl = OW::getPluginManager()->getPlugin('helpcenter')->getStaticUrl();
		$themeStaticUrl = OW::getThemeManager()->getCurrentTheme()->getStaticUrl();

		$document->addStyleSheet( $pluginStaticUrl . 'css/bootstrap-reboot.min.css', 'all', -100 );
		$document->addStyleSheet( $pluginStaticUrl . 'css/bootstrap-grid.min.css', 'all', -99 );
		$document->addStyleSheet( $pluginStaticUrl . 'css/bootstrap.min.css', 'all', -96 );

		$document->addStyleSheet(OW::getPluginManager()->getPlugin('base')->getStaticCssUrl() . 'ow.css' . '?' . OW::getConfig()->getValue('base', 'cachedEntitiesPostfix'), 'all', -90);
        $document->addStyleSheet($themeManager->getCssFileUrl() . '?' . OW::getConfig()->getValue('base', 'cachedEntitiesPostfix'), 'all', (-90));
		
		if ( $themeManager->getCurrentTheme()->getDto()->getCustomCssFileName() !== null )
		{
			$document->addStyleSheet($themeManager->getThemeService()->getCustomCssFileUrl($themeManager->getCurrentTheme()->getDto()->getKey()));
		}
		
		$document->addStyleSheet( $service->getCurrentThemeUrl() . 'base.css' );
		$document->addStyleDeclaration($service->getThemeCustomCss());

		$document->addCustomHeadInfo(' ');
		
		$document->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'jquery.min.js',
            'text/javascript', (-100));
        $document->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'jquery-migrate.min.js',
            'text/javascript', (-100));
        $document->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'ow.js?' . OW::getConfig()->getValue('base', 'cachedEntitiesPostfix'), 'text/javascript', (-50));
		
		$addActionUrl = OW::getRouter()->urlFor('HELPCENTER_CTRL_View', 'vote');
		
		OW::getLanguage()->addKeyForJs('helpcenter', 'vote_thanks');

		$jsGenerator = UTIL_JsGenerator::newInstance();
		$jsGenerator->setVariable('window.helpcenterRateActionUrl', $addActionUrl);

		$variables = $jsGenerator->generateJs();
		$document->addScriptDeclaration($variables);
		
		$document->addScript($pluginStaticUrl . 'js/bootstrap.min.js');
		$document->addScript($pluginStaticUrl . 'js/bootstrap.bundle.min.js');
		$document->addScript($pluginStaticUrl . 'js/script.js');
	}

    public function onBeforeRender()
    {
        parent::onBeforeRender();
		
		$document = OW::getDocument();
		
		$document->setStyleSheets(array(
			'added' => array(),
			'items' => array(),
		));
		
		$document->setJavaScripts(array(
			'added' => array(),
			'items' => array(),
		));
		
		$this->loadStaticFiles();

		$document->addBodyClass('helpcenter');
		$this->setTemplate(OW::getPluginManager()->getPlugin('helpcenter')->getCtrlViewDir() . 'template.html');
    }
}