<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2018, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Ebenezer Obasi <info@eobasi.com>
 * @package ow_plugins.helpcenter.controllers
 * @since 1.0
 */
class HELPCENTER_CTRL_Topic extends ADMIN_CTRL_Abstract
{
	public function __construct( )
	{
		parent::__construct( );

		$this->lang = OW::getLanguage();
		$this->service = HELPCENTER_BOL_Service::getInstance();

		$this->addComponent( 'menu', $this->service->getAdminMenu() );
	}

	public function index( )
	{
		$this->setPageTitle($this->lang->text('helpcenter', 'admin_topic_page_title'));
		$this->setPageHeading($this->lang->text('helpcenter', 'admin_topic_page_heading'));

		$tagService = BOL_TagService::getInstance();
		$voteService = BOL_VoteService::getInstance();

		$topics = $this->service->findAllTopic();

		foreach($topics as $topic)
		{
			$topicId = $topic->getId( );
			$category = $this->service->findCategory( $topic->getCategory( ) );

			$tags = $tagService->findEntityTags($topicId, 'helpcenter');

			$topic->category = $category->name;
			$topic->tags = $tags;
			$topic->vote = (int)$voteService->findTotalVotesResult( $topicId, 'helpcenter' )['sum'];
			$topic->viewUrl = OW::getRouter()->urlForRoute('helpcenter_view_topic', array('id' => $topicId));
			$topic->deleteUrl = OW::getRouter()->urlFor(__CLASS__, 'delete', array('type' => 'topic', 'id' => $topicId));
			$topic->editUrl = OW::getRouter()->urlForRoute('helpcenter_admin_add', array('id' => $topicId));
		}

		$this->assign('topics', $topics);
		$this->assign('url_new_post',  OW::getRouter()->urlForRoute('helpcenter_admin_add_new'));
		$this->assign('deleteConfirmMsg', $this->lang->text('admin', 'are_you_sure') );
	}

	public function add( $params )
	{
		$id = isset( $params['id'] ) ? $params['id'] : 'new';

		$form = new HELPCENTER_CLASS_TopicForm();

		if( $id != 'new' )
		{
			$topic = $this->service->findTopic( $id );

			if( empty( $topic ) )
			{
				throw new Redirect404Exception();
			}

			$this->setPageTitle($this->lang->text('helpcenter', 'admin_edit_page_title'));
			$this->setPageHeading($this->lang->text('helpcenter', 'admin_edit_page_heading'));

			$form = new HELPCENTER_CLASS_TopicForm( $topic );
			$topicId = $topic->getId();
		}
		else
		{
			$this->setPageTitle($this->lang->text('helpcenter', 'admin_add_page_title'));
			$this->setPageHeading($this->lang->text('helpcenter', 'admin_add_page_heading'));
		}

		if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
		{
			$data = $form->getValues();

			$topicDto = new HELPCENTER_BOL_Topic();

			$topicDto->setTitle( $data['title'] );
			$topicDto->setBody( $data['body'] );
			$topicDto->setCategory( $data['category'] );
			$topicDto->setTimeStamp( );
			$topicDto->setStatus( true );

			if( isset( $topicId ) )
			{
				$topicDto->setId( $topicId );
				$this->service->saveTopic( $topicDto );
			}
			else
			{
				$topicId = $this->service->saveTopic( $topicDto );
			}

			$tagService = BOL_TagService::getInstance();
			$tags = array();

			if ( intval($topicId) > 0 )
			{
				$tags = $data['tags'];
			}

			$tagService->updateEntityTags($topicId, 'helpcenter', $tags);

			OW::getFeedback()->info($this->lang->text('helpcenter', 'admin_topic_saved'));
			$this->redirect(OW::getRouter()->urlForRoute('helpcenter_admin_add', array('id' => $topicId)));
		}

		$this->addForm($form);
	}

	public function category( )
	{
		$this->setPageTitle($this->lang->text('helpcenter', 'admin_category_page_title'));
		$this->setPageHeading($this->lang->text('helpcenter', 'admin_category_page_heading'));

		$form = new HELPCENTER_CLASS_CategoryForm();

		if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
		{
			$data = $form->getValues();

			$category = new HELPCENTER_BOL_Category();
			$category->setName( $data['name'] );
			$category->setDescription( $data['description'] );
			$category->setParent( $data['parent'] );

			$this->service->saveCategory( $category );
			OW::getFeedback()->info($this->lang->text('helpcenter', 'admin_category_added'));
			$this->redirect();
		}

		$this->addForm($form);

		$categories = $this->service->findAllCategory( true );

		foreach($categories as $category)
		{
			$category->count = count($this->service->categoryTopics( $category->getId( ) ));
			$category->desc = !empty($category->getDescription( )) ? $category->getDescription( ) : '—';
			$category->url = $this->url( $category->getId(), 'category' );

			foreach($category->children as $child)
			{
				$child->count = count($this->service->categoryTopics( $child->getId( ) ));
				$child->desc = !empty($child->getDescription( )) ? $child->getDescription( ) : '—';
				$child->url = $this->url( $child->getId(), 'category' );

				foreach($child->children as $granChild)
				{
					$granChild->count = count($this->service->categoryTopics( $granChild->getId( ) ));
					$granChild->desc = !empty($granChild->getDescription( )) ? $granChild->getDescription( ) : '—';
					$granChild->url = $this->url( $granChild->getId(), 'category' );
				}
			}
		}

		$this->assign('categories', $categories);
		$this->assign('deleteConfirmMsg', $this->lang->text('admin', 'are_you_sure') );
	}

	public function setting( )
	{
		$config = OW::getConfig();
		$categories = HELPCENTER_BOL_Service::getInstance()->findAllCategory( true );

		$this->setPageTitle($this->lang->text('helpcenter', 'admin_setting_page_title'));
		$this->setPageHeading($this->lang->text('helpcenter', 'admin_setting_page_heading'));

		$form = new Form('helpcenter_settings');

		$field = new CheckboxField('display_faq');
		$field->setValue($this->service->config->display_faq);
		$field->setLabel($this->lang->text('helpcenter', 'admin_setting_display_faq_field_label'));
		$form->addElement($field);

		$field = new Selectbox('featured_count');
		$field->setLabel($this->lang->text('helpcenter', 'admin_setting_featured_count_field_label'));
		for( $i = 5; $i <= 20; $i++ ) $field->addOption($i, $i);
		$field->setValue($this->service->config->featured_count);
		$form->addElement($field);

		$field = new Selectbox('default_category');
		$field->setLabel($this->lang->text('helpcenter', 'admin_setting_default_category_field_label'));
		foreach( $categories as $category )
		{
			$field->addOption($category->id, $category->name);

			foreach($category->children as $child)
			{
				$field->addOption($child->id, '— '.$child->name);
			}
		}
		$field->setValue($this->service->config->default_category);
		$form->addElement($field);

		$field = new Selectbox('theme');
		$field->setLabel($this->lang->text('helpcenter', 'admin_setting_theme_field_label'));
		$field->setOptions($this->service->getThemes());
		$field->setValue($this->service->config->theme);
		$form->addElement($field);

		$field = new HELPCENTER_CLASS_Wysiwyg('custom_index_content');
		$field->setSize(HELPCENTER_CLASS_Wysiwyg::SIZE_S);
        $field->setLabel($this->lang->text("helpcenter", "admin_setting_custom_index_content_field_label"));
		$field->setValue($this->service->config->custom_index_content);
		$form->addElement($field);

		$field = new Textarea('css');
        $field->setLabel($this->lang->text("helpcenter", "admin_setting_css_field_label"));
        $field->setHasInvitation(true);
        $field->setInvitation($this->lang->text('helpcenter', 'admin_setting_css_field_invitation'));
		$field->setValue($this->service->getThemeCustomCss());
		$form->addElement($field);

		$buttonSave = new Submit('save');
		$buttonSave->setValue($this->lang->text('helpcenter', 'admin_button_save_value'));
		$form->addElement($buttonSave);

		$this->addForm($form);

		if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
		{
			$data = $form->getValues();

			$config->saveConfig('helpcenter', 'display_faq', trim($data['display_faq']));
			$config->saveConfig('helpcenter', 'default_category', trim($data['default_category']));
			$config->saveConfig('helpcenter', 'featured_count', trim($data['featured_count']));
			$config->saveConfig('helpcenter', 'theme', trim($data['theme']));
			$config->saveConfig('helpcenter', 'custom_index_content', trim($data['custom_index_content']));
			$config->saveConfig('helpcenter', 'css', $this->service->setThemeCustomCss( trim($data['css']) ));

			OW::getFeedback()->info($this->lang->text('helpcenter', 'admin_settings_updated'));
			$this->redirect();
		}

		$this->addForm($form);

	}

	public function delete( $params )
	{
		$id = isset( $params['id'] ) ? $params['id'] : null;
		$type = isset( $params['type'] ) ? $params['type'] : null;

		if ( empty( $id ) || empty( $type ) )
        {
            throw new Redirect404Exception();
        }

		switch( $type )
		{
			case 'category':
				$category = $this->service->findCategory( $id );

				if( empty( $category ) )
				{
					throw new Redirect404Exception();
				}

				$this->service->deleteCategory( $id );
				OW::getFeedback()->info($this->lang->text('helpcenter', 'admin_category_deleted'));
				$this->redirect( OW::getRouter()->urlForRoute( 'helpcenter_admin_category' ) );
				break;
			case 'topic':
				$topic = $this->service->findTopic( $id );

				if( empty( $topic ) )
				{
					throw new Redirect404Exception();
				}

				$this->service->deleteTopic( $id );
				OW::getFeedback()->info($this->lang->text('helpcenter', 'admin_topic_deleted'));
				$this->redirect( OW::getRouter()->urlForRoute( 'helpcenter_admin_index' ) );
				break;
			default:
				throw new Redirect404Exception();
		}
	}

	private function url( $id, $type )
	{
		return $this->service->itemUrl( $id, $type );
	}
}