<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2018, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Ebenezer Obasi <info@eobasi.com>
 * @package ow_plugins.helpcenter.controllers
 * @since 1.0
 */
class HELPCENTER_CTRL_View extends OW_ActionController
{
	public function __construct( )
	{
		parent::__construct( );

		$this->lang = OW::getLanguage();
		$this->service = HELPCENTER_BOL_Service::getInstance();

		OW::getDocument()->setMasterPage(new HELPCENTER_CLASS_MasterPage( ) );
	}

	public function index( )
	{
		$this->assign('content', $this->service->config->custom_index_content);
	}

    public function vote()
    {
        if ( !($userId = OW::getUser()->getId()) )
        {
            echo json_encode(array('isAuthenticated' => false, 'msg' => OW::getLanguage()->text('helpcenter', 'sign_in_request')));
            exit;
        }

        $id = $_POST['itemId'];

        $voteService = BOL_VoteService::getInstance();

        $vote = $voteService->findUserVote($id, 'helpcenter', $userId);
        $voteType = 0;

        switch ( $_POST['vote'] )
        {
            case 'YES':
                if ( !$vote )
                {
                    $vote = new BOL_Vote();
                    $vote->setEntityType('helpcenter')->setEntityId($id)->setUserId($userId);
                }
                $voteType = 1;
                $vote->setVote($voteType);
                $voteType = "+1";
                $voteService->saveVote($vote->setTimeStamp(time()));
                break;

            case 'NO':
                if ( !$vote )
                {
                    $vote = new BOL_Vote();
                    $vote->setEntityType('helpcenter')->setEntityId($id)->setUserId($userId);
                }
                $voteType = -1;
                $vote->setVote($voteType);
                $voteType = "-1";
                $voteService->saveVote($vote->setTimeStamp(time()));

                break;

            case 'CANCEL':

                if ( $vote !== null )
                {
                    $voteService->delete($vote);
                }

                break;

            default:
                echo '{}';
                exit;
        }


        $total = $voteService->findTotalVotesResult($id, 'helpcenter');

        echo json_encode((object) array('type' => 'helpcenter', 'id' => $id, 'total' => $total, 'voteType' => $voteType));

        exit;
    }

	public function board( $param )
	{
		if( empty( $param['catId'] ) )
		{
			throw new Redirect404Exception();
		}

		$id = $param['catId'];
		$category = $this->service->findCategory( $id );

		if( empty( $category ) )
		{
			throw new Redirect404Exception();
		}

		$sidebar = new HELPCENTER_CMP_Sidebar( $id );
		$this->addComponent( 'sidebar', $sidebar );

		$topics = $this->service->categoryTopics( $id );
		$this->assign('topics', $topics);

	}

	public function topic( $params )
	{
		if( empty( $params['id'] ) )
		{
			throw new Redirect404Exception();
		}

		$id = $params['id'];
		$topic = $this->service->findTopic( $id );

		if( empty( $topic ) )
		{
			throw new Redirect404Exception();
		}

		$sidebar = new HELPCENTER_CMP_Sidebar( $topic->getCategory() );

		$this->addComponent( 'sidebar', $sidebar );
		$this->assign( 'topic', $topic );
	}

	public function search( )
	{
		$term = isset( $_GET['search'] ) ? $_GET['search'] : '';
		
		if( empty( $term ) )
		{
			$this->index();
			$this->setTemplate(OW::getPluginManager()->getPlugin('helpcenter')->getCtrlViewDir() . 'view_index.html');
			
			return;
		}
		
		$results = $this->service->searchTopic( $term );
		
		if( empty( $results ) )
		{
			$siteName = OW::getConfig()->getValue('base', 'site_name');
			$siteUrl = OW::getRouter()->getBaseUrl();
			
			$this->assign('siteName', $siteName);		
			$this->assign('siteUrl', $siteUrl);		
			$this->assign('term', $term);		
		
			 $this->setTemplate(OW::getPluginManager()->getPlugin('helpcenter')->getCtrlViewDir() . 'not_found.html');
			 return;
		}
		
		$this->assign('results', $results);
	}
}